@@ -1,4 +1,4 @@
-from django.utils.encoding import smart_unicode, force_unicode, smart_str
+from django.utils.encoding import smart_unicode, force_unicode
 from django.utils.text import capfirst
 from django.utils import formats
 from django.core.exceptions import ObjectDoesNotExist
@@ -11,7 +11,7 @@ import copy
 
 from serializer import PRIMITIVE_PROCESSOR
 from exceptions import DotPathNotFound
-from common import get_schema, DotPathList, DotPathDict, DotPathSet, UnSet
+from common import DotPathList, DotPathDict, DotPathSet, UnSet
 from dockit.forms.fields import HiddenSchemaField, HiddenListField, HiddenDictField, SchemaChoiceField, PrimitiveListField, SchemaMultipleChoiceField
 
 class NOT_PROVIDED:
@@ -85,7 +85,10 @@ class BaseField(object):
         as SelectField choices for this field."""
         first_choice = include_blank and blank_choice or []
         if self.choices:
-            return first_choice + list(self.choices)
+            choices = self.choices
+            if callable(choices):
+                choices = choices()
+            return first_choice + list(choices)
         return first_choice
         #rel_model = self.rel.to
         #if hasattr(self.rel, 'get_related_field'):
@@ -166,6 +169,17 @@ class BaseField(object):
         obj = copy.copy(self)
         memodict[id(self)] = obj
         return obj
+    
+    def _get_val_from_obj(self, obj):
+        return obj[self.name]
+    
+    def value_to_string(self, obj):
+        val = self._get_val_from_obj(obj)
+        return self.to_primitive(val)
+    
+    @property
+    def attname(self):
+        return self.name
 
 class BaseTypedField(BaseField):
     coerce_function = None
@@ -369,13 +383,13 @@ class SchemaField(BaseField):
         else:
             parent[attr] = value
 
-class GenericSchemaField(BaseField):
+class GenericSchemaField(BaseField): #meant to be sublcassed
     def __init__(self, field_name='_type', **kwargs):
         self.type_field_name = field_name
         super(GenericSchemaField, self).__init__(**kwargs)
     
     def lookup_schema(self, key):
-        return get_schema(key)
+        raise NotImplementedError
     
     def get_schema_type(self, val):
         return val._meta.schema_key
@@ -675,13 +689,6 @@ class ReferenceField(BaseField):
     def _meta(self):
         return self.document._meta
     
-    def contribute_to_class(self, cls, name):
-        new_field = copy.copy(self)
-        if self.self_reference and not cls._meta.virtual:
-            new_field.document = cls
-            new_field.self_reference = False
-        BaseField.contribute_to_class(new_field, cls, name)
-    
     def is_instance(self, val):
         if val is None:
             return True
@@ -695,10 +702,18 @@ class ReferenceField(BaseField):
         return val.get_id()
     
     def to_python(self, val, parent=None):
-        if self.is_instance(val):
-            return val
+        if self.self_reference:
+            if val is None:
+                return val
+            document = type(parent)
+            if isinstance(val, document):
+                return val
+        else:
+            if self.is_instance(val):
+                return val
+            document = self.document
         try:
-            return self.document.objects.get(pk=val)
+            return document.objects.get(pk=val)
         except ObjectDoesNotExist:
             if self.null:
                 return None
