@@ -1,174 +1,29 @@
-from dockit.backends import get_document_backend
-
-import re
 import sys
 
-from django.conf import settings
-from django.db.models.options import get_verbose_name
-from django.utils.translation import activate, deactivate_all, get_language, string_concat
-from django.utils.encoding import smart_str, force_unicode
+from django.utils.encoding import force_unicode
 from django.utils.datastructures import SortedDict
 from django.db.models import FieldDoesNotExist
+from django.db.models.loading import app_cache_ready
 from django.core.exceptions import ObjectDoesNotExist, MultipleObjectsReturned
 
 from manager import Manager
-from common import register_schema, DotPathTraverser, UnSet
+from common import register_collection, DotPathTraverser, UnSet
 from signals import pre_save, post_save, pre_delete, post_delete, class_prepared, pre_init, post_init
+import options
 
 def subclass_exception(name, parents, module):
     return type(name, parents, {'__module__': module})
 
-class Options(object):
-    """ class based on django.db.models.options. We only keep
-    useful bits."""
-    
-    abstract = False
-    ordering = ['_id']
-    
-    DEFAULT_NAMES = ('verbose_name', 'db_table', 'ordering', 'schema_key',
-                     'app_label', 'collection', 'virtual', 'proxy',
-                     'typed_field', 'typed_key')
-    
-    def __init__(self, meta, app_label=None):
-        self.module_name, self.verbose_name = None, None
-        self.verbose_name_plural = None
-        self.object_name, self.app_label = None, app_label
-        self.meta = meta
-        self.fields = SortedDict()
-        self.collection = None
-        self.schema_key = None
-        self.virtual = False #TODO all schemas are virtual
-        self.proxy = False
-        self._document = None
-        self.typed_field = None
-        self.typed_key = None
-    
-    def process_values(self, cls):
-        cls._meta = self
-        self._document = cls
-        self.installed = re.sub('\.models$', '', cls.__module__) in settings.INSTALLED_APPS
-        # First, construct the default values for these options.
-        self.object_name = cls.__name__
-        self.module_name = self.object_name.lower()
-        self.verbose_name = get_verbose_name(self.object_name)
-        self.collection = self.default_schema_key()
-        self.schema_key = self.default_schema_key()
-
-        # Next, apply any overridden values from 'class Meta'.
-        if getattr(self, 'meta', None):
-            meta_attrs = self.meta.__dict__.copy()
-            for name in self.meta.__dict__:
-                # Ignore any private attributes that Django doesn't care about.
-                # NOTE: We can't modify a dictionary's contents while looping
-                # over it, so we loop over the *original* dictionary instead.
-                if name.startswith('_'):
-                    del meta_attrs[name]
-            for attr_name in self.DEFAULT_NAMES:
-                if attr_name in meta_attrs:
-                    setattr(self, attr_name, meta_attrs.pop(attr_name))
-                elif hasattr(self.meta, attr_name):
-                    setattr(self, attr_name, getattr(self.meta, attr_name))
-
-            # verbose_name_plural is a special case because it uses a 's'
-            # by default.
-            setattr(self, 'verbose_name_plural', meta_attrs.pop('verbose_name_plural', string_concat(self.verbose_name, 's')))
-
-            # Any leftover attributes must be invalid.
-            if meta_attrs != {}:
-                raise TypeError("'class Meta' got invalid attribute(s): %s" % ','.join(meta_attrs.keys()))
-            del self.meta
-        else:
-            self.verbose_name_plural = string_concat(self.verbose_name, 's')
-    
-    def contribute_to_class(self, cls, name):
-        #the following needs to happen after fields are populated
-        if self.typed_field:
-            if self.typed_key:
-                if self.virtual:
-                    raise TypeError("Virtual Schemas may not have a typed_key")
-                #CONSIDER this will break if fields are registered after the meta, gennerally have the virtual create the field for you
-                if self.typed_field not in self.fields:
-                    raise TypeError("Non-virtual Schemas that specify a typed field and a typed key must have that typed field defined.")
-                self.fields[self.typed_field].schemas[self.typed_key] = cls
-                self.proxy = True
-            else:
-                #if not self.virtual:
-                #    raise TypeError("Schemas that specify a typed_field and not a typed_key must be virtual.")
-                if self.typed_field not in self.fields:
-                    from fields import SchemaTypeField
-                    self.polymorphic_schemas = SortedDict()
-                    field = SchemaTypeField(self.polymorphic_schemas, editable=False)
-                    field.contribute_to_class(cls, self.typed_field)
-    
-    def default_schema_key(self):
-        return "%s.%s" % (smart_str(self.app_label), smart_str(self.module_name))
-    
-    def __str__(self):
-        return self.collection
-
-    def verbose_name_raw(self):
-        """
-        There are a few places where the untranslated verbose name is needed
-        (so that we get the same value regardless of currently active
-        locale).
-        """
-        lang = get_language()
-        deactivate_all()
-        raw = force_unicode(self.verbose_name)
-        activate(lang)
-        return raw
-    verbose_name_raw = property(verbose_name_raw)
-    
-    def get_field(self, name):
-        if name not in self.fields:
-            raise FieldDoesNotExist
-        return self.fields[name]
-    
-    def get_field_by_name(self, name):
-        """
-        Returns the (field_object, model, direct, m2m), where field_object is
-        the Field instance for the given name, model is the model containing
-        this field (None for local fields), direct is True if the field exists
-        on this model, and m2m is True for many-to-many relations. When
-        'direct' is False, 'field_object' is the corresponding RelatedObject
-        for this field (since the field doesn't have an instance associated
-        with it).
-
-        Uses a cache internally, so after the first access, this is very fast.
-        """
-        if name not in self.fields:
-            raise FieldDoesNotExist
-        return (self.fields[name], None, True, False)
-    
-    def get_ordered_objects(self):
-        return []
-    
-    @property
-    def pk(self):
-        class DummyField(object):
-            def __init__(self, **kwargs):
-                for key, value in kwargs.iteritems():
-                    setattr(self, key, value)
-        return DummyField(attname='pk')
-    
-    def get_backend(self):
-        return get_document_backend()
-    
-    def dot_notation_to_field(self, notation):
-        traverser = DotPathTraverser(notation)
-        traverser.resolve_for_schema(self._document)
-        return traverser.current['field']
-    
-    def is_dynamic(self):
-        return bool(self.typed_field)
-
 class SchemaBase(type):
     """
     Metaclass for all schemas.
     """
+    options_module = options.SchemaOptions
+    
     def __new__(cls, name, bases, attrs):
         super_new = super(SchemaBase, cls).__new__
         parents = [b for b in bases if isinstance(b, SchemaBase)]
+        options_module = cls.options_module
         
         module = attrs.pop('__module__')
         new_class = super_new(cls, name, bases, {'__module__': module})
@@ -183,7 +38,7 @@ class SchemaBase(type):
                 if not hasattr(new_class, '_meta'):
                     raise ValueError('Proxy schemas must inherit from another schema')
                 parent_meta = getattr(new_class, '_meta')
-                for key in Options.DEFAULT_NAMES:
+                for key in options_module.DEFAULT_NAMES:
                     if not hasattr(meta, key) and hasattr(parent_meta, key):
                         setattr(meta, key, getattr(parent_meta, key))
         
@@ -193,14 +48,15 @@ class SchemaBase(type):
         else:
             app_label = getattr(meta, 'app_label')
         
+        parent_fields = list()
         for base in bases:
             if hasattr(base, '_meta') and hasattr(base._meta, 'fields'):
-                attrs.update(base._meta.fields)
+                parent_fields.append(base._meta.fields)
         
         fields = [(field_name, attrs.pop(field_name)) for field_name, obj in attrs.items() if hasattr(obj, 'creation_counter')]
         fields.sort(key=lambda x: x[1].creation_counter)
         
-        options = Options(meta, app_label=app_label)
+        options = options_module(meta, app_label=app_label, parent_fields=parent_fields)
         options.process_values(new_class)
         setattr(new_class, '_meta', options)
         
@@ -212,9 +68,6 @@ class SchemaBase(type):
         
         new_class.add_to_class('_meta', options)
         
-        if not new_class._meta.virtual:
-            register_schema(new_class._meta.schema_key, new_class)
-        
         class_prepared.send(**{'sender':cls, 'class':new_class})
         return new_class
     
@@ -381,7 +234,16 @@ class Schema(object):
         else:
             self[attr] = value
 
+_pending_registered_documents = list()
+
+def _register_document(document_cls):
+    backend = document_cls._meta.get_backend()
+    backend.register_document(document_cls)
+    register_collection(document_cls)
+
 class DocumentBase(SchemaBase):
+    options_module = options.DocumentOptions
+    
     def __new__(cls, name, bases, attrs):
         new_class = SchemaBase.__new__(cls, name, bases, attrs)
         if 'objects' not in attrs:
@@ -389,8 +251,7 @@ class DocumentBase(SchemaBase):
             objects.contribute_to_class(new_class, 'objects')
         
         if not new_class._meta.virtual and not new_class._meta.proxy:
-            backend = get_document_backend()
-            backend.register_document(new_class)
+            _register_document(new_class)
         
         parents = [b for b in bases if isinstance(b, DocumentBase)]
         module = new_class.__module__
@@ -414,6 +275,9 @@ class Document(Schema):
         backend = self._meta.get_backend()
         return backend.get_id(self._primitive_data)
     
+    def _get_pk_val(self):
+        return self.get_id()
+    
     pk = property(get_id)
     
     def save(self):
@@ -456,15 +320,19 @@ class UserMeta(object):
         for key, value in kwargs.iteritems():
             setattr(self, key, value)
 
-def create_schema(name, fields, module='dockit.models', base=SchemaBase, parents=(Schema,)):
-    attrs = SortedDict(fields)
-    attrs['__module__'] = module
-    return base.__new__(base, name, parents, attrs)
+def create_schema(name, fields, attrs={}, module='dockit.models', base=SchemaBase, parents=(Schema,), **kwargs):
+    all_attrs = SortedDict(fields)
+    all_attrs.update(attrs)
+    all_attrs['__module__'] = module
+    if kwargs:
+       all_attrs['Meta'] = UserMeta(**kwargs)
+    return base.__new__(base, name, parents, all_attrs)
 
-def create_document(name, fields, module='dockit.models', collection=None, virtual=False, base=DocumentBase, parents=(Document,)):
-    attrs = SortedDict(fields)
-    attrs['__module__'] = module
-    if collection:
-       attrs['Meta'] = UserMeta(collection=collection)
-    return base.__new__(base, name, parents, attrs)
+def create_document(name, fields, attrs={}, module='dockit.models', base=DocumentBase, parents=(Document,), **kwargs):
+    all_attrs = SortedDict(fields)
+    all_attrs.update(attrs)
+    all_attrs['__module__'] = module
+    if kwargs:
+       all_attrs['Meta'] = UserMeta(**kwargs)
+    return base.__new__(base, name, parents, all_attrs)
 
